@@ -1,9 +1,9 @@
-#line 1 "/tmp/pikedeb.20bb6f0000/7.4/src/interpret_functions.h"
+#line 1 "/tmp/pikedeb.5a120871e4/7.4/src/interpret_functions.h"
 /*
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: interpret_functions.h,v 1.129 2004/03/13 15:59:46 grubba Exp $
+|| $Id: interpret_functions.h,v 1.130 2004/10/06 18:35:02 mast Exp $
 */
 
 /*
@@ -258,7 +258,7 @@ OPCODE1_TAIL(F_MARK_AND_STRING, "mark & string", 0, {
     print_return_value();
   };
 })
-#line 252 "/tmp/pikedeb.20bb6f0000/7.4/src/interpret_functions.h"
+#line 252 "/tmp/pikedeb.5a120871e4/7.4/src/interpret_functions.h"
 OPCODE1(F_STRING, "string", 0, {
     copy_shared_string(Pike_sp->u.string,Pike_fp->context.prog->strings[arg1]);
     Pike_sp->type=PIKE_T_STRING;
@@ -266,7 +266,7 @@ OPCODE1(F_STRING, "string", 0, {
     Pike_sp++;
     print_return_value();
   })
-#line 259 "/tmp/pikedeb.20bb6f0000/7.4/src/interpret_functions.h"
+#line 259 "/tmp/pikedeb.5a120871e4/7.4/src/interpret_functions.h"
 
 
 
@@ -360,7 +360,7 @@ OPCODE1_TAIL(F_MARK_AND_GLOBAL, "mark & global", 0, {
     print_return_value();
   };
 })
-#line 344 "/tmp/pikedeb.20bb6f0000/7.4/src/interpret_functions.h"
+#line 344 "/tmp/pikedeb.5a120871e4/7.4/src/interpret_functions.h"
 OPCODE1(F_GLOBAL, "global", 0, {
     low_object_index_no_free(Pike_sp,
 			     Pike_fp->current_object,
@@ -368,7 +368,7 @@ OPCODE1(F_GLOBAL, "global", 0, {
     Pike_sp++;
     print_return_value();
   })
-#line 351 "/tmp/pikedeb.20bb6f0000/7.4/src/interpret_functions.h"
+#line 351 "/tmp/pikedeb.5a120871e4/7.4/src/interpret_functions.h"
 
 
 OPCODE2_TAIL(F_MARK_AND_EXTERNAL, "mark & external", 0, {
@@ -402,7 +402,7 @@ OPCODE2_TAIL(F_MARK_AND_EXTERNAL, "mark & external", 0, {
     print_return_value();
   };
 })
-#line 356 "/tmp/pikedeb.20bb6f0000/7.4/src/interpret_functions.h"
+#line 356 "/tmp/pikedeb.5a120871e4/7.4/src/interpret_functions.h"
 OPCODE2(F_EXTERNAL,"external", 0, {
     struct external_variable_context loc;
 
@@ -430,7 +430,7 @@ OPCODE2(F_EXTERNAL,"external", 0, {
     }
     print_return_value();
   })
-#line 383 "/tmp/pikedeb.20bb6f0000/7.4/src/interpret_functions.h"
+#line 383 "/tmp/pikedeb.5a120871e4/7.4/src/interpret_functions.h"
 
 
 
@@ -587,6 +587,7 @@ OPCODE1(F_INC_LOCAL, "++local", 0, {
       )
   {
     push_int(++(Pike_fp->locals[arg1].u.integer));
+    Pike_fp->locals[arg1].subtype = NUMBER_NUMBER; /* Could have UNDEFINED there before. */
   } else {
     push_svalue(Pike_fp->locals+arg1);
     push_int(1);
@@ -605,6 +606,7 @@ OPCODE1(F_POST_INC_LOCAL, "local++", 0, {
       )
   {
     Pike_fp->locals[arg1].u.integer++;
+    Pike_fp->locals[arg1].subtype = NUMBER_NUMBER; /* Could have UNDEFINED there before. */
   } else {
     push_svalue(Pike_fp->locals + arg1);
     push_int(1);
@@ -621,6 +623,7 @@ OPCODE1(F_INC_LOCAL_AND_POP, "++local and pop", 0, {
       )
   {
     Pike_fp->locals[arg1].u.integer++;
+    Pike_fp->locals[arg1].subtype = NUMBER_NUMBER; /* Could have UNDEFINED there before. */
   } else {
     push_svalue( Pike_fp->locals + arg1);
     push_int(1);
@@ -637,6 +640,7 @@ OPCODE1(F_DEC_LOCAL, "--local", 0, {
       )
   {
     push_int(--(Pike_fp->locals[arg1].u.integer));
+    Pike_fp->locals[arg1].subtype = NUMBER_NUMBER; /* Could have UNDEFINED there before. */
   } else {
     push_svalue(Pike_fp->locals+arg1);
     push_int(1);
@@ -655,6 +659,7 @@ OPCODE1(F_POST_DEC_LOCAL, "local--", 0, {
       )
   {
     Pike_fp->locals[arg1].u.integer--;
+    Pike_fp->locals[arg1].subtype = NUMBER_NUMBER; /* Could have UNDEFINED there before. */
   } else {
     push_svalue(Pike_fp->locals + arg1);
     push_int(1);
@@ -672,6 +677,7 @@ OPCODE1(F_DEC_LOCAL_AND_POP, "--local and pop", 0, {
       )
   {
     Pike_fp->locals[arg1].u.integer--;
+    Pike_fp->locals[arg1].subtype = NUMBER_NUMBER; /* Could have UNDEFINED there before. */
   } else {
     push_svalue(Pike_fp->locals + arg1);
     push_int(1);
@@ -753,6 +759,7 @@ OPCODE0(F_ADD_TO_AND_POP, "+= and pop", 0, {
     {
       /* Optimization for a rather common case. Makes it 30% faster. */
       Pike_sp[-1].u.integer += Pike_sp[-2].u.integer;
+      Pike_sp[-1].subtype = NUMBER_NUMBER; /* Could have UNDEFINED there before. */
       assign_lvalue(Pike_sp-4,Pike_sp-1);
       Pike_sp-=2;
       pop_2_elems();
@@ -1031,18 +1038,18 @@ OPCODE0_TAIL(F_MARK2, "mark mark", 0, {
     };
   };
 })
-#line 976 "/tmp/pikedeb.20bb6f0000/7.4/src/interpret_functions.h"
+#line 983 "/tmp/pikedeb.5a120871e4/7.4/src/interpret_functions.h"
 OPCODE0_TAIL(F_SYNCH_MARK, "synch mark", 0, {
 
     {
       *(Pike_mark_sp++)=Pike_sp;
     };
   })
-#line 978 "/tmp/pikedeb.20bb6f0000/7.4/src/interpret_functions.h"
+#line 985 "/tmp/pikedeb.5a120871e4/7.4/src/interpret_functions.h"
 OPCODE0(F_MARK, "mark", 0, {
       *(Pike_mark_sp++)=Pike_sp;
     })
-#line 982 "/tmp/pikedeb.20bb6f0000/7.4/src/interpret_functions.h"
+#line 989 "/tmp/pikedeb.5a120871e4/7.4/src/interpret_functions.h"
 
 
 OPCODE1(F_MARK_X, "mark Pike_sp-X", 0, {
@@ -1079,7 +1086,7 @@ OPCODE0_TAIL(F_CLEANUP_SYNCH_MARK, "cleanup synch mark", 0, {
     }
   };
 })
-#line 1000 "/tmp/pikedeb.20bb6f0000/7.4/src/interpret_functions.h"
+#line 1007 "/tmp/pikedeb.5a120871e4/7.4/src/interpret_functions.h"
 OPCODE0(F_POP_SYNCH_MARK, "pop synch mark", 0, {
     if (d_flag) {
       if (Pike_mark_sp <= Pike_interpreter.mark_stack) {
@@ -1097,7 +1104,7 @@ OPCODE0(F_POP_SYNCH_MARK, "pop synch mark", 0, {
       }
     }
   })
-#line 1017 "/tmp/pikedeb.20bb6f0000/7.4/src/interpret_functions.h"
+#line 1024 "/tmp/pikedeb.5a120871e4/7.4/src/interpret_functions.h"
 
 
 OPCODE0(F_CLEAR_STRING_SUBTYPE, "clear string subtype", 0, {
@@ -1131,7 +1138,7 @@ OPCODE2_BRANCH(F_BRANCH_IF_NOT_LOCAL_ARROW, "branch if !local->x", 0, {
     pop_stack();
   };
 })
-#line 1040 "/tmp/pikedeb.20bb6f0000/7.4/src/interpret_functions.h"
+#line 1047 "/tmp/pikedeb.5a120871e4/7.4/src/interpret_functions.h"
 OPCODE0_TAILBRANCH(F_BRANCH_WHEN_ZERO, "branch if zero", 0, {
     if(!UNSAFE_IS_ZERO(Pike_sp-1))
     {
@@ -1141,7 +1148,7 @@ OPCODE0_TAILBRANCH(F_BRANCH_WHEN_ZERO, "branch if zero", 0, {
     }
     pop_stack();
   })
-#line 1049 "/tmp/pikedeb.20bb6f0000/7.4/src/interpret_functions.h"
+#line 1056 "/tmp/pikedeb.5a120871e4/7.4/src/interpret_functions.h"
 
 
       
@@ -1416,16 +1423,21 @@ OPCODE0_BRANCH(F_LOOP, "loop", 0, { /* loopcnt */
   }
 })
 
-OPCODE0_BRANCH(F_FOREACH, "foreach", 0, { /* array, lvalue, X, i */
+OPCODE0_BRANCH(F_FOREACH, "foreach", 0, { /* array, lvalue, i */
   if(Pike_sp[-4].type != PIKE_T_ARRAY)
     PIKE_ERROR("foreach", "Bad argument 1.\n", Pike_sp-3, 1);
   if(Pike_sp[-1].u.integer < Pike_sp[-4].u.array->size)
   {
     if(Pike_sp[-1].u.integer < 0)
+      /* Isn't this an internal compiler error? /mast */
       Pike_error("Foreach loop variable is negative!\n");
     assign_lvalue(Pike_sp-3, Pike_sp[-4].u.array->item + Pike_sp[-1].u.integer);
     DO_BRANCH();
     Pike_sp[-1].u.integer++;
+    DO_IF_DEBUG (
+      if (Pike_sp[-1].subtype)
+	Pike_fatal ("Got unexpected subtype in loop variable.\n");
+    );
   }else{
     DONT_BRANCH();
   }
@@ -1518,8 +1530,11 @@ OPCODE0(F_NEGATE, "unary minus", 0, {
 	o_negate();
       }
       else
-      )
+    )
+    {
       Pike_sp[-1].u.integer =- Pike_sp[-1].u.integer;
+      Pike_sp[-1].subtype = NUMBER_NUMBER; /* Could have UNDEFINED there before. */
+    }
   }
   else if(Pike_sp[-1].type == PIKE_T_FLOAT)
   {
@@ -1536,6 +1551,7 @@ OPCODE0(F_NOT, "!", 0, {
   {
   case PIKE_T_INT:
     Pike_sp[-1].u.integer =! Pike_sp[-1].u.integer;
+    Pike_sp[-1].subtype = NUMBER_NUMBER; /* Could have UNDEFINED there before. */
     break;
 
   case PIKE_T_FUNCTION:
@@ -1553,6 +1569,7 @@ OPCODE0(F_NOT, "!", 0, {
   default:
     free_svalue(Pike_sp-1);
     Pike_sp[-1].type=PIKE_T_INT;
+    Pike_sp[-1].subtype = NUMBER_NUMBER;
     Pike_sp[-1].u.integer=0;
   }
 })
@@ -1586,6 +1603,7 @@ OPCODE0(F_ADD_INTS, "int+int", 0, {
     )
   {
     Pike_sp[-2].u.integer+=Pike_sp[-1].u.integer;
+    Pike_sp[-2].subtype = NUMBER_NUMBER; /* Could have UNDEFINED there before. */
     Pike_sp--;
   }else{
     f_add(2);
@@ -1618,6 +1636,7 @@ OPCODE1(F_ADD_INT, "add integer", 0, {
      )
   {
     Pike_sp[-1].u.integer+=arg1;
+    Pike_sp[-1].subtype = NUMBER_NUMBER; /* Could have UNDEFINED there before. */
   }else{
     push_int(arg1);
     f_add(2);
@@ -1632,6 +1651,7 @@ OPCODE1(F_ADD_NEG_INT, "add -integer", 0, {
      )
   {
     Pike_sp[-1].u.integer-=arg1;
+    Pike_sp[-1].subtype = NUMBER_NUMBER; /* Could have UNDEFINED there before. */
   }else{
     push_int(-arg1);
     f_add(2);
@@ -2297,11 +2317,11 @@ OPCODE1_JUMP(F_COND_RECUR, "recur if not overloaded", 0, {
     DO_RECUR(0);
   };
 })
-#line 2200 "/tmp/pikedeb.20bb6f0000/7.4/src/interpret_functions.h"
+#line 2220 "/tmp/pikedeb.5a120871e4/7.4/src/interpret_functions.h"
 OPCODE0_TAILJUMP(F_RECUR, "recur", 0, {
     DO_RECUR(0);
   })
-#line 2203 "/tmp/pikedeb.20bb6f0000/7.4/src/interpret_functions.h"
+#line 2223 "/tmp/pikedeb.5a120871e4/7.4/src/interpret_functions.h"
 
 
 /* Ugly code duplication */
