@@ -1,9 +1,16 @@
+/*
+|| This file is part of Pike. For copyright information see COPYRIGHT.
+|| Pike is distributed under GPL, LGPL and MPL. See the file COPYING
+|| for more information.
+|| $Id: iso2022.c,v 1.27 2003/09/23 17:53:52 mast Exp $
+*/
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif /* HAVE_CONFIG_H */
 
 #include "global.h"
-RCSID("$Id: iso2022.c,v 1.21 2003/09/23 17:53:42 mast Exp $");
+RCSID("$Id: iso2022.c,v 1.27 2003/09/23 17:53:52 mast Exp $");
 #include "program.h"
 #include "interpret.h"
 #include "stralloc.h"
@@ -13,8 +20,9 @@ RCSID("$Id: iso2022.c,v 1.21 2003/09/23 17:53:42 mast Exp $");
 
 #include "iso2022.h"
 
-/* MUST BE LAST INCLUDE */
-#include "module_magic.h"
+
+#define sp Pike_sp
+#define fp Pike_fp
 
 #define PRGM_NAME "Locale.Charset.ISO2022"
 
@@ -434,6 +442,7 @@ static void eat_enc_string(struct pike_string *str, struct iso2022enc_stor *s,
     {
       char *p = str->str;
       p_wchar2 c;
+
       while(l--) {
 	if(s1) {
 	  c = *(p_wchar1 *)p;
@@ -635,7 +644,7 @@ static void eat_enc_string(struct pike_string *str, struct iso2022enc_stor *s,
     }
     break;
   default:
-    fatal("Illegal shift size!\n");
+    Pike_fatal("Illegal shift size!\n");
   }
 }
 
@@ -700,14 +709,13 @@ static void f_enc_feed(INT32 args)
   struct pike_string *str;
 
   get_all_args(PRGM_NAME"Enc->feed()", args, "%W", &str);
-
-  eat_enc_string(str, (struct iso2022enc_stor *)fp->current_storage,
-		 ((struct iso2022enc_stor *)fp->current_storage)->replace,
-		 (((struct iso2022enc_stor *)fp->current_storage)->repcb.type
-		  == T_FUNCTION?
-		  &((struct iso2022enc_stor *)fp->current_storage)->repcb :
-		  NULL));
-
+  if( str->len )
+    eat_enc_string(str, (struct iso2022enc_stor *)fp->current_storage,
+		   ((struct iso2022enc_stor *)fp->current_storage)->replace,
+		   (((struct iso2022enc_stor *)fp->current_storage)->repcb.type
+		    == T_FUNCTION?
+		    &((struct iso2022enc_stor *)fp->current_storage)->repcb :
+		    NULL));
   pop_n_elems(args);
   push_object(this_object());
 }
