@@ -1,12 +1,9 @@
-/*\
-||| This file a part of Pike, and is copyright by Fredrik Hubinette
-||| Pike is distributed as GPL (General Public License)
-||| See the files COPYING and DISCLAIMER for more information.
-\*/
-
 /*
- * $Id: file.h,v 1.20 2003/09/03 11:38:07 mast Exp $
- */
+|| This file is part of Pike. For copyright information see COPYRIGHT.
+|| Pike is distributed under GPL, LGPL and MPL. See the file COPYING
+|| for more information.
+|| $Id: file.h,v 1.27 2003/12/09 09:57:14 grubba Exp $
+*/
 
 #ifndef FILE_H
 #define FILE_H
@@ -61,7 +58,7 @@ struct pike_sendfile
 {
   struct object *self;
 
-  ptrdiff_t sent;
+  LONGEST sent;
 
   struct array *headers;
   struct array *trailers;
@@ -79,8 +76,8 @@ struct pike_sendfile
   struct my_file *from;
   struct my_file *to;
 
-  ptrdiff_t offset;
-  ptrdiff_t len;
+  LONGEST offset;
+  LONGEST len;
 
   struct iovec *hd_iov;
   struct iovec *tr_iov;
@@ -127,13 +124,11 @@ struct new_thread_data;
 void file_proxy(INT32 args);
 void create_proxy_pipe(struct object *o, int for_reading);
 struct file_lock_key_storage;
-void pike_module_exit(void);
 void mark_ids(struct callback *foo, void *bar, void *gazonk);
-void pike_module_init(void);
 int pike_make_pipe(int *fds);
 int fd_from_object(struct object *o);
 void f_strerror(INT32 args);
-void push_stat(struct stat *s);
+void push_stat(PIKE_STAT_T *s);
 /* Prototypes end here */
 
 #define FILE_READ               0x1000
