@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: svalue.c,v 1.160 2004/01/12 12:26:54 marcus Exp $
+|| $Id: svalue.c,v 1.163 2005/09/09 15:09:08 mast Exp $
 */
 
 #include "global.h"
@@ -72,7 +72,7 @@ static int pike_isnan(double x)
 #define PIKE_ISUNORDERED(X,Y) (PIKE_ISNAN(X)||PIKE_ISNAN(Y))
 #endif /* HAVE_ISUNORDERED */
 
-RCSID("$Id: svalue.c,v 1.160 2004/01/12 12:26:54 marcus Exp $");
+RCSID("$Id: svalue.c,v 1.163 2005/09/09 15:09:08 mast Exp $");
 
 struct svalue dest_ob_zero = {
   T_INT, 0,
@@ -781,6 +781,7 @@ PMOD_EXPORT int is_eq(const struct svalue *a, const struct svalue *b)
   switch(a->type)
   {
   case T_OBJECT:
+    if (a->u.object == b->u.object) return 1;
     if(FIND_LFUN(a->u.object->prog,LFUN_EQ) != -1)
       goto a_is_obj;
 
@@ -1159,6 +1160,10 @@ PMOD_EXPORT void describe_svalue(const struct svalue *s,int indent,struct proces
 {
   char buf[50];
 
+  /* This needs to be a bit lower than LOW_C_STACK_MARGIN so that the
+   * the raw error can be printed in exit_on_error. */
+  check_c_stack(250);
+
   check_type(s->type);
   check_refs(s);
 
